using System;
using System.Configuration;
using System.Globalization;
using System.IO;
using System.Xml.Serialization;
using MailBee;
using MailBee.Mime;

namespace WebMailLite
{
	/// <summary>
	/// Summary description for FileSystem.
	/// </summary>
	public class FileSystem
	{
		protected string _rootFolder;
		protected string _accountName;
		protected int _accountID = 0;
		protected int _accountHierarchyDepth = 1;

		public string RootFolder
		{
			get { return _rootFolder; }
			set { _rootFolder = value; }
		}

		public string AccountName
		{
			get { return _accountName; }
			set { _accountName = value; }
		}

		public FileSystem(string accountName, int accountID, bool isMailFolder) : this(GetDataFolderFromSettings(isMailFolder), accountName, accountID)
		{
		}

		public FileSystem(string rootFolder, string accountName, int accountID)
		{
			_rootFolder = rootFolder;
			_accountName = accountName;
			_accountID = accountID;
		}

		public void CreateAccount()
		{
			WebmailResourceManager resMan = (new WebmailResourceManagerCreator()).CreateResourceManager();

			if (!IsFolderNameValid(_accountName))
			{
				throw new WebMailIOException(resMan.GetString("CantCreateAccount"));
			}

			CreateFolder("");
		}

		/// <summary>
		/// Create folder on disk
		/// </summary>
		/// <param name="folder"></param>
		/// <returns>Created folder full path.</returns>
		public string CreateFolder(string folder)
		{
			try
			{
				string folderToCreate = CreateFolderFullPath(folder);
				if (!Directory.Exists(folderToCreate))
				{
					Directory.CreateDirectory(folderToCreate);
				}
				return folderToCreate;
			}
			catch (WebMailIOException)
			{
				throw;
			}
			catch (ArgumentException ex)
			{
				throw new WebMailIOException(ex);
			}
			catch (UnauthorizedAccessException ex)
			{
				throw new WebMailIOException(ex);
			}
			catch (IOException ex)
			{
				throw new WebMailIOException(ex);
			}
		}

		public bool SaveAccount(Account acct, string hashString)
		{
			if (acct == null || hashString == string.Empty) 
			{
				return false;
			}

            try
			{
				string filepath = Utils.GetTempFolderNameFromAcct(acct.Email);

				XmlSerializer s = new XmlSerializer(typeof(Account));
				StreamWriter w = new StreamWriter(filepath + "/" + hashString);
				s.Serialize(w, acct);
				w.Close();
			}
			catch
			{
				return false;
			}
			return true;
		}

		public Account ReadAccount(string email, string hashString)
		{
			Account acct = null;
			if (email == string.Empty || hashString == string.Empty)
			{
				return acct;
			}

			try
			{
				string filepath = Utils.GetTempFolderNameFromAcct(email);
				XmlSerializer s = new XmlSerializer(typeof(Account));
				FileStream r = new FileStream(filepath + @"\" + hashString, FileMode.Open);
				acct = (Account) s.Deserialize(r);
				r.Close();
			}
			catch(Exception ex)
			{
				Log.WriteLine("FileStream or Deserialize", ex.Message);
				return null;
			}
			return acct;
		}

		public bool DeleteAccount(string email, string hashString)
		{
			if (email == string.Empty || hashString == string.Empty) 
			{
				return false;
			}

			try
			{
				string fullpath = Utils.GetTempFolderNameFromAcct(email) + @"\" + hashString;
				File.Delete(fullpath);
			}
			catch(Exception ex)
			{
				Log.WriteLine("File.Delete", ex.Message);
				return false;
			}
			return true;
		}

        public static bool DeleteAccountTempFolder(string tempFolder)
        {
            if (null == tempFolder) return false;

            try
            {
                if (Directory.Exists(tempFolder))
                {
                    Directory.Delete(tempFolder, true);
                }
            }
            catch (Exception ex)
            {
                Log.WriteLine("Directory.Delete", ex.Message);
                return false;
            }
            return true;
        }

        public static bool DeleteTempFolder(string settingsDataFolderPath)
        {
            try
            {
                Log.WriteLine("DeleteTempFolder", "start");
                string fullpath = Path.Combine(settingsDataFolderPath, Constants.tempFolderName);
                Log.WriteLine("DeleteTempFolder fullpath", fullpath);
                if (Directory.Exists(fullpath))
                {
                    Log.WriteLine("Directory.Exists", "true");
                    Directory.Delete(fullpath, true);
                    Log.WriteLine("Directory.Delete", "end");
                }
            }
            catch (Exception ex)
            {
                Log.WriteLine("Directory.Delete", ex.Message);
                return false;
            }
            return true;
        }

		protected string CreateFolderFullPath(string folder)
		{
			Log.WriteLine("CreateFolderFullPath", folder);
			folder = Utils.ConvertToUtf7Modified(folder);
			try
			{
				string returnPath = string.Empty;

				if (_rootFolder != null) returnPath = _rootFolder;

				if (_accountName != null)
				{
					EmailAddress adr = new EmailAddress();
					adr.AsString = _accountName;
					string account = adr.GetAccountName();

					for (int i = 0; i <= _accountHierarchyDepth - 1; i++)
					{
						if (i >= account.Length) break;
						returnPath = Path.Combine(returnPath, account[i].ToString());
					}
					returnPath = Path.Combine(returnPath, string.Format("{0}.{1}", _accountName, _accountID));
				}

				Log.WriteLine("CreateFolderFullPath", Path.Combine(returnPath, folder));
				return Path.Combine(returnPath, folder);
			}
			catch (ArgumentException ex)
			{
				throw new WebMailIOException(ex);
			}
		}

		protected static string GetDataFolderFromSettings(bool isMailFolder)
		{
			string folder = (isMailFolder) ? Constants.mailFolderName : Constants.tempFolderName;
			string dataFolder = Utils.GetDataFolderPath();
			if (dataFolder != null) 
				return Path.Combine(dataFolder, folder);
			else
				return folder;
		}

		public static bool IsFolderNameValid(string folderName)
		{
			if (folderName == null) return false;
			string[] invalidNames = new string[] { "CON", "AUX", "COM1", "COM2", "COM3", "COM4", "LPT1", "LPT2", "LPT3", "PRN", "NUL"};
			foreach (string invalidName in invalidNames)
			{
				if (string.Compare(folderName, invalidName, true, CultureInfo.InvariantCulture) == 0)
				{
					return false;
				}
			}
			if (folderName.IndexOfAny(new char[] {'"', '/', '\\', '*', '?', '<', '>', '|', ':'}) >= 0)
			{
				return false;
			}
			return true;
		}


	}
}
