using System;
using System.Collections;
using System.Configuration;
using System.Globalization;
using System.IO;
using MailBee;
using MailBee.Mime;
using MailBee.Pop3Mail;
using MailBee.Security;
using System.Threading;

namespace WebMailLite
{
	/// <summary>
	/// Summary description for Pop3Storage.
	/// </summary>
	public class Pop3Storage : MailServerStorage
	{
		protected Pop3 _pop3Obj;

		public Pop3Storage(Account account) : base(account)
		{
			WebmailSettings settings = (new WebMailSettingsCreator()).CreateWebMailSettings();

			try
			{
				Pop3.LicenseKey = settings.LicenseKey;
				_pop3Obj = new Pop3();
				_pop3Obj.MessageDownloaded += new Pop3MessageDownloadedEventHandler(_pop3Obj_MessageDownloaded);

				if (settings.EnableLogging)
				{
					_pop3Obj.Log.Enabled = true;
					string dataFolderPath = Utils.GetDataFolderPath();
					if (dataFolderPath != null)
					{
						_pop3Obj.Log.Filename = Path.Combine(dataFolderPath, Constants.logFilename);
					}
				}
			}
			catch (MailBeeException ex)
			{
				throw new WebMailMailBeeException(ex);
			}
		}

		public override void Connect(bool login)
		{
			WebmailResourceManager resMan = (new WebmailResourceManagerCreator()).CreateResourceManager();
			try
			{
                if (login)
                {
                    _pop3Obj.InboxPreloadOptions = Pop3InboxPreloadOptions.None;
                }
                else
                {
                    _pop3Obj.InboxPreloadOptions = Pop3InboxPreloadOptions.Uidl;
                }
				//---------------- special for gmail -------------
				if (_account.MailIncomingPort == 995)
				{
					_pop3Obj.SslMode = SslStartupMode.OnConnect;
					_pop3Obj.SslProtocol = SecurityProtocol.Auto;
					_pop3Obj.SslCertificates.AutoValidation = CertificateValidationFlags.None;
				}
				//------------------------------------------------
				_pop3Obj.Connect(_account.MailIncomingHost, _account.MailIncomingPort, true);
				_pop3Obj.Login(_account.MailIncomingLogin, _account.MailIncomingPassword, AuthenticationMethods.Auto, AuthenticationOptions.PreferSimpleMethods, null);
			}
			catch (MailBeeConnectionException)
			{
				throw new WebMailException(resMan.GetString("ErrorPOP3Connect"));
			}
			catch (MailBeePop3LoginBadCredentialsException)
			{
				throw new WebMailException(resMan.GetString("ErrorPOP3IMAP4Auth"));
			}
			catch(MailBeeException ex)
			{
				throw new WebMailMailBeeException(ex);
			}
		}

		public override bool IsConnected()
		{
			return _pop3Obj.IsConnected;
		}

		public override void Disconnect()
		{
			try
			{
				if (IsConnected()) _pop3Obj.Disconnect();
			}
			catch (MailBeeException ex)
			{
				throw new WebMailMailBeeException(ex);
			}
		}

		public override long GetMailStorageSize()
		{
			return _pop3Obj.InboxSize;
		}

		public override int GetMessageCount()
		{
			return _pop3Obj.InboxMessageCount;
		}

		public override MailMessageCollection LoadMessageHeaders(int pageNumber)
		{
			int[] indexes = GetIndexesFromPageNumber(pageNumber);
			MailMessageCollection coll = LoadMessageHeaders(indexes);
			CollectionBase cb = coll as CollectionBase;
			if (cb != null)
			{
				Utils.ReverseCollection(ref cb);
			}
			return coll;
		}

		public MailMessageCollection LoadMessageHeaders(int[] messageIndexSet)
		{
			MailMessageCollection msgsColl = new MailMessageCollection();
			foreach (int index in messageIndexSet)
			{
				if (index > 0)
				{
					try
					{
						MailMessage msg = _pop3Obj.DownloadMessageHeader(index);
						msgsColl.Add(msg);
					}
					catch(MailBeeException ex)
					{
						throw new WebMailMailBeeException(ex);
					}
				}
				else
				{
					continue;
				}
			}
			return msgsColl;
		}

		public override MailMessage LoadMessage(string uid)
		{
			int indexOnServer = _pop3Obj.GetMessageIndexFromUid(uid);
            System.Threading.Thread.Sleep(100);
            try
			{
				if (indexOnServer > 0)
				{
					return _pop3Obj.DownloadEntireMessage(indexOnServer);
				}
				else
				{
					throw new WebMailMailBoxException((new WebmailResourceManagerCreator()).CreateResourceManager().GetString("PROC_MSG_HAS_DELETED"));
				}
			}
			catch (MailBeeException ex)
			{
				throw new WebMailMailBeeException(ex);
			}
		}

		public override void DeleteMessages(string[] messageUidSet)
		{
			foreach (string uid in messageUidSet)
			{
				int index = _pop3Obj.GetMessageIndexFromUid(uid);
                System.Threading.Thread.Sleep(100);
				try
				{
					if (index > 0)
					{
						OnMessageDeleted(new DeleteMessageEventArgs(index, "Inbox"));
						_pop3Obj.DeleteMessage(index);
					}
				}
				catch (MailBeeException ex)
				{
					throw new WebMailMailBeeException(ex);
				}
			}
		}

		public override int GetInboxMessageCount(string fullPath)
		{
			return _pop3Obj.InboxMessageCount;
		}

		public override int GetInboxUnreadMessageCount(string fullPath)
		{
			return 0;
		}

		private int[] GetIndexesFromPageNumber(int pageNumber)
		{
			WebmailSettings settings = (new WebMailSettingsCreator()).CreateWebMailSettings();
			int msgPerPage = settings.MailsPerPage;
			int msgCount = _pop3Obj.InboxMessageCount;
			
			msgPerPage = settings.MailsPerPage;
			
			int startIndex = 1;
			int length = msgPerPage;
			if ((pageNumber * msgPerPage) > msgCount)
			{
				if (msgCount > msgPerPage)
				{
					length = msgCount % msgPerPage;
				}
				else
				{
					length = msgCount;
				}
			}
			else
			{
				startIndex = (msgCount - (pageNumber * msgPerPage)) + 1;
			}

			//object[] ids = new object[(end - start) + 1];
			ArrayList arr = new ArrayList();
			for (int i = 0; i < length; i++)
			{
				if (((startIndex + i) <= _pop3Obj.InboxMessageCount) && ((startIndex + i) > 0))
				{
					arr.Add(startIndex + i);
				}
			}
			return (int[])arr.ToArray(typeof(int));
		}

		private void _pop3Obj_MessageDownloaded(object sender, Pop3MessageDownloadedEventArgs e)
		{
			OnMessageDownloaded(new CheckMailEventArgs(_folderName, _msgsCount, _msgNumber));
			_msgNumber++;
		}
	}
}